<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of DotClear.
# Copyright (c) 2004 Olivier Meunier and contributors. All rights
# reserved.
#
# DotClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# DotClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#eze
# ***** END LICENSE BLOCK *****
#
# Contributor : Aurelien ANTOINE (http://ilpleut.be/)
#

require_once (dirname(__FILE__).'/convert/ConvertCharset.class.php');

class MPTools {
	
	var $bitags;
	var $base_bitags;
	var $monotags;
	var $convert;
	var $charset;
	var $savepath;
	var $preg_attr;
	var $nbr_attached_file;
	var $attached_file;
	var $attached_filename;
	var $nbr_inline_file;
	var $cid;
	var $inline_file;
	var $inline_filename;
	var $text_version;
	var $text_encoding;
	var $html_version;
	var $html_encoding;
	
	function MPTools($charset, $convert, $savepath, $relpath) {
		#UTF-8/ISO-8859-1 and iconv/soft
		
		$this->convert=$convert;
		$this->charset=$charset;
		$this->savepath=$savepath;
		$this->relpath=$relpath;
		
		#gen param for preg in iso and utf8
		if ($this->charset=='UTF-8') {
			$this->preg_attr='siu';
		}else{
			$this->preg_attr='si';
		}
		#define some var used to cleanup html
		$this->bitags=array(
			'a'		=> array("href", "title", "type"),
			'blockquote'	=> array("cite","title","dir"),
			'del'		=> array("cite", "title","dir","datetime"),
			'ins'		=> array("cite", "title","dir","datetime"),
			'q'		=> array("cite","title","dir"),
			'table'		=> array("border", "cellpadding", "cellspacing", "dir", "frame", "rules", "summary", "title", "width"),
			'tbody'		=> array("align", "char", "charoff", "dir", "title", "valign"),
			'td'		=> array("abbr", "align", "axis", "char", "charoff", "colspan", "dir", "rowspan", "title", "valign"),
			'tfoot'		=> array("align", "char", "charoff", "dir", "title", "valign"),
			'th'		=> array("abbr", "align", "axis", "char", "charoff", "colspan", "dir", "rowspan", "title", "valign"),
			'thead'		=> array("align", "char", "charoff", "dir", "title", "valign"),
			'tr'		=> array("align", "char", "charoff", "dir", "title", "valign"),
			
		);
		
		$this->base_bitags=array('abbr', 'acronym', 'address', 'b', 'big', 'caption',
			'cite', 'code', 'dd', 'dfn', 'dl', 'dt', 'em', 'fieldset', 'h1', 'h2', 'h3', 
			'h4', 'h5', 'h6', 'i', 'kbd', 'legend', 'li', 'ol', 'p', 'pre', 'samp', 
			'small', 'strong', 'sub', 'sup', 'ul', 'var'
		);
		
		$this->monotags=array(
			'img' 		=> array("alt", "height", "width", "src", "dir", "longdesc", "title"),
			'br'		=> array(),
			'hr'		=> array(),
		);
		
	}

	#to display object in html way 
	function print_h($var){
		ob_start();
			print_r($var);
			$texte=ob_get_contents();
		ob_end_clean();
		echo $this->txt2html($texte);
	}
	
	#convert txt to html
	function txt2html($texte) {
		$texte=str_replace(" ","&nbsp;",$texte);
		$texte=str_replace("<","&lt;",$texte);
		return nl2br($texte);
	}
	
	#convert html to ascii
	function html2ascii($texte) {
		if ($this->charset=='UTF-8') {
			$texte=utf8_decode($texte);
		}
		$texte=strip_tags($texte);
		$texte=strtr($texte,"","eeeiacu");
		$from=array(
			"%&nbsp;%",
			"%&amp;%",
			"%&([a-z])[a-z]*?;%",
			"%\s+%",
		);
		$to=array(" ","&","\$1"," ");
		$texte=preg_replace($from, $to, $texte);
		return $texte;
	}

	#clean up a tag (callback function)
	function cleantag_callback($tag) {
		#extract params, and take them as vars
		//echo $tag[1].$tag[2]."<br>";
		if (preg_match_all("%([a-z]+)\s*=\s*(?:\"([^\"]*)\"|'([^']*)'|([a-z0-9]+))%".$this->preg_attr,$tag[2],$result)) {
			for($i=0;$i<count($result[2]);$i++) {
				if($result[2][$i]!="") $value=$result[2][$i];
				elseif($result[3][$i]!="") $value=$result[3][$i];
				else $value=$result[4][$i];
				$name=strtolower($result[1][$i]);
				$$name=$value;
			}
		}
		
		$tag=strtolower($tag[1]);
		
		#detect if it's a close tag
		if (preg_match("%^/(.*)%",$tag,$result)) {
			$close_tag=$result[1];
		}else{
			$close_tag="";
		}
		
		if (array_key_exists($tag, $this->bitags)) {
			$res='<'.$tag;
			foreach($this->bitags[$tag] as $arg) {
				if ($$arg!="") $res.=' '.$arg.'="'.$$arg.'"';
			}
			$res.='>';
		}elseif(array_key_exists($close_tag,$this->bitags)) {
			$res='<'.$tag.'>';
			
		}elseif(in_array($tag, $this->base_bitags)) {
			$res='<'.$tag;
			if ($title!="") $res.=' title="'.$title.'"';
			if ($dir!="") $res.=' dir="'.$dir.'"';
			$res.='>';
		}elseif(in_array($close_tag, $this->base_bitags)) {
			$res='<'.$tag.'>';
			
		}elseif(array_key_exists($tag, $this->monotags)) {
			if ($tag=='img' && $alt=='') $alt=' ';
			$res='<'.$tag;
			foreach($this->monotags[$tag] as $arg) {
				if ($$arg!="") $res.=' '.$arg.'="'.$$arg.'"';
			}
			$res.=' />';
		}
		return $res;
	}

	#to clean an html code (hard way)
	function cleanup_html($html) {
		#remove some portion of code
		$pattern=array(
			'%<script[^>]*?>.*?</script[^>]*?>%'.$this->preg_attr,
			'%<style[^>]*?>.*?</style[^>]*?>%'.$this->preg_attr,
			'%<head[^>]*?>.*?</head[^>]*?>%'.$this->preg_attr,
			'%<title[^>]*?>.*?</title[^>]*?>%'.$this->preg_attr,
			'%<!\-\-.*?\-\->%'.$this->preg_attr,
		);
		
		$replace=array(
			'',
			'',
			'',
			'',
			'',
		);
		
		$html = preg_replace($pattern,$replace,$html);
		
		#clean some tag, and remove others
		$lines=preg_split("%(<[^>]*?>)%",$html,-1,PREG_SPLIT_DELIM_CAPTURE);
		$html="";
		foreach($lines as $line) {
			if (preg_match("%<(/?[a-z\-\!:]+)([^>]*)>%".$this->preg_attr,$line,$result)) {
				$html.=$this->cleantag_callback($result);
			}else{
				$line=htmlentities($line,ENT_QUOTES,$this->charset);
				$html.=str_replace("amp;","",$line); #we don't change the &, in case of already encoded ...
			}
		}
		
		#clean the \r\n stuff
		$html = preg_replace("%(\s*\r?\n\s*)+%".$this->preg_attr,"\r\n",$html);
		
		#remove <br /> at begining and end if any
		$html = preg_replace('%(<br />\s*)+$%',"",$html);
		$html = preg_replace('%^(\s*<br />)+%',"",$html);
		
		return $html;
	}
	
	#convert encoding 
	function convert($txt,$input,$output='') {
		if ($output=='') $output=$this->charset;
		if ($this->convert=='iconv') {
			return $this->convert_iconv($txt, $input, $output);
		}else{
			return $this->convert_soft($txt, $input, $output);
		}
	}
	
	function convert_iconv($txt, $in, $out) {
		$txt = iconv($in, $out, $txt);
		return $txt;
	}
	
	function convert_soft($txt, $in, $out) {
		$convert = new ConvertCharset;
		$txt = $convert->Convert($txt, $in, $out, FALSE);
		return $txt;
	}
	
	#init all params for a mail
	function init_mail_data() {
		$this->nbr_attached_file=0;
		$this->attached_file=array();
		$this->attached_filename=array();
		$this->nbr_inline_file=0;
		$this->cid=array();
		$this->inline_file=array();
		$this->inline_filename=array();
		$this->text_version='';
		$this->text_encoding='';
		$this->html_version='';
		$this->html_encoding='';
		$this->subject='';
		$this->date='';
	}
	
	function get_header_data($tree) {
		$this->date=strtotime($tree->headers[date]);
		$this->subject=$this->convert($tree->headers[subject],'ISO-8859-1');
	}	
	
	#parse the mail structure and get all info
	function parse_tree($tree) {
		if ($tree->ctype_primary=='multipart') {
			if ($tree->ctype_secondary=='mixed') {
				//we have attached files
				foreach($tree->parts as $parts) {
					if ($parts->d_parameters[filename]!="") {
						$this->attached_file[$this->nbr_attached_file]=$parts->d_parameters['filename'];
						$this->attached_filename[$this->nbr_attached_file]=md5($parts->body).$parts->d_parameters['filename'];
						$fptr=@fopen($this->savepath.$this->attached_filename[$this->nbr_attached_file], 'wb');
						@fwrite($fptr, $parts->body );
						@fclose($fptr);
						$this->attached_abs_filename[$this->nbr_attached_file]=$this->savepath.$this->attached_filename[$this->nbr_attached_file];
						$this->attached_filename[$this->nbr_attached_file]=$this->relpath.$this->attached_filename[$this->nbr_attached_file];
						$this->nbr_attached_file++;
					}else{
						$this->parse_tree($parts);
					}
				}
			}elseif ($tree->ctype_secondary=='related') {
				foreach($tree->parts as $parts) {
					if ($parts->d_parameters[filename]!="" && $parts->ctype_primary=='image') {
						$this->cid[$this->nbr_inline_file]="cid:".preg_replace("%[<> ]%","",$parts->headers['content-id']);
						$this->inline_file[$this->nbr_inline_file]=$parts->d_parameters['filename'];
						$this->inline_filename[$this->nbr_inline_file]=md5($parts->body).$parts->d_parameters['filename'];
						$fptr=@fopen($this->savepath.$this->inline_filename[$this->nbr_inline_file], 'wb');
						@fwrite($fptr, $parts->body );
						@fclose($fptr);
						$this->inline_abs_filename[$this->nbr_inline_file]=$this->savepath.$this->inline_filename[$this->nbr_inline_file];
						$this->inline_filename[$this->nbr_inline_file]=$this->relpath.$this->inline_filename[$this->nbr_inline_file];
						$this->nbr_inline_file++;
					}else{
						$this->parse_tree($parts);
					}
				}
				
			}else{
				foreach($tree->parts as $parts) {
					$this->parse_tree($parts);
				}
			}
		}elseif ($tree->ctype_primary=='text') {
			if ($tree->ctype_secondary=='plain' && $tree->d_parameters[filename]=='' && $this->text_version=='' ) {
				$this->text_version=$tree->body;
				$this->text_encoding=preg_replace("%;.*$%","",$tree->ctype_parameters[charset]);
				#echo "text version:<br>".$this->text_version."<br>";
				#echo "text encoding: ".$this->text_encoding."<br>";
			}elseif ($tree->ctype_secondary=='html' && $tree->d_parameters[filename]=='' && $this->html_version=='' ) {
				$this->html_version=$tree->body;
				$this->html_encoding=preg_replace("%;.*$%","",$tree->ctype_parameters[charset]);
				#echo "html version:<br>".$this->html_version."<br>";
				#echo "html encoding: ".$this->html_encoding."<br>";
			}
		}
	}
	
	#take the html or text version
	function htmlortxt() {
		if(trim($this->html_version)=='' && trim($this->text_version)!='') {
			$this->html_version=$this->txt2html($this->text_version);
			$this->html_encoding=$this->text_encoding;
		}
	}
	
	#convert cid image to browser format
	function check_image($txt) {
		return str_replace($this->cid,$this->inline_filename,$txt);
	}
	
	#add attached files at the end of body
	function add_attached_file_list($txt) {
		if ($this->nbr_attached_file>0) {
			$txt.="<br /><ul>";
			for($i=0;$i<$this->nbr_attached_file;$i++) {
				$txt.='<li><a href="'.$this->attached_filename[$i].'">'.$this->attached_file[$i].'</a></li>';
			}
			$txt.="</ul><br />";
		}
		return $txt;
	}
}
